/*
 *  jDTAUS - DTAUS fileformat.
 *  Copyright (c) 2005 Christian Schulte <cs@schulte.it>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */
package org.jdtaus.core.container.ri.servlet;

import java.io.IOException;
import java.util.Locale;
import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.FilterConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import org.jdtaus.core.container.ContainerFactory;
import org.jdtaus.core.container.Context;
import org.jdtaus.core.container.ContextFactory;
import org.jdtaus.core.container.ModelFactory;

/**
 * jDTAUS webapp integration filter.
 * <p>This filter needs to be setup in each application's {@code web.xml}
 * descriptor like:<pre>
 * &lt;filter&gt;
 *   &lt;filter-name&gt;jDTAUS Integration Filter&lt;/filter-name&gt;
 *   &lt;filter-class&gt;org.jdtaus.core.container.ri.servlet.ServletFilter&lt;/filter-class&gt;
 * &lt;/filter&gt;</pre></p>
 *
 * @author <a href="mailto:cs@schulte.it">Christian Schulte</a>
 * @version $Id: ServletFilter.java 3527 2007-05-17 04:05:40Z schulte2005 $
 *
 * @see #init(FilterConfig)
 */
public class ServletFilter implements Filter
{
    //--ServletFilter-----------------------------------------------------------

    /** Holds thread local {@code ServletContext}s. */
    private static final ThreadLocal CONTEXTS = new ThreadLocal();

    /** Holds thread local {@code HttpSession}s. */
    private static final ThreadLocal SESSIONS = new ThreadLocal();

    /** Holds the {@code ServletContext} the filter got initialized with. */
    private ServletContext servletContext;

    /**
     * Gets the thread local servlet context.
     *
     * @return the thread local servlet context.
     *
     * @throws ContextLostException if no servlet context is bound to the
     * current thread of execution.
     */
    public static ServletContext getServletContext()
    {
        final ServletContext context =
            (ServletContext) ServletFilter.CONTEXTS.get();

        if(context == null)
        {
            throw new ContextLostException(Thread.currentThread());
        }

        return context;
    }

    /**
     * Gets the thread local HTTP session.
     *
     * @return the thread local HTTP session.
     *
     * @throws SessionLostException if no session is bound to the current
     * thread of execution or if the requesting {@code ServletRequest} is not
     * an instance of {@code HttpServletRequest}.
     */
    public static HttpSession getHttpSession()
    {
        final HttpSession session = (HttpSession) ServletFilter.SESSIONS.get();
        if(session == null)
        {
            throw new SessionLostException(Thread.currentThread());
        }

        return session;
    }

    //-----------------------------------------------------------ServletFilter--
    //--Filter------------------------------------------------------------------

    /**
     * Initializes the filter by configuring {@code ContainerFactory},
     * {@code ModelFactory} and {@code ContextFactory} to use the
     * {@code ServletContextFactories} implementation.
     *
     * @see ServletContextFactories
     */
    public void init(FilterConfig filterConfig) throws ServletException
    {
        this.servletContext = filterConfig.getServletContext();

        // Initialize system properties if nothing is provided.
        if(System.getProperty(ContainerFactory.class.getName()) == null)
        {
            System.setProperty(ContainerFactory.class.getName(),
                ServletContextFactories.class.getName());

            filterConfig.getServletContext().log(ServletFilterBundle.
                getSystemPropertyWarningMessage(Locale.getDefault()).
                format(new Object[] { ContainerFactory.class.getName(),
                ServletContextFactories.class.getName() }));

        }
        if(System.getProperty(ContextFactory.class.getName()) == null)
        {
            System.setProperty(ContextFactory.class.getName(),
                ServletContextFactories.class.getName());

            filterConfig.getServletContext().log(ServletFilterBundle.
                getSystemPropertyWarningMessage(Locale.getDefault()).
                format(new Object[] { ContextFactory.class.getName(),
                ServletContextFactories.class.getName() }));

        }
        if(System.getProperty(ModelFactory.class.getName()) == null)
        {
            System.setProperty(ModelFactory.class.getName(),
                ServletContextFactories.class.getName());

            filterConfig.getServletContext().log(ServletFilterBundle.
                getSystemPropertyWarningMessage(Locale.getDefault()).
                format(new Object[] { ModelFactory.class.getName(),
                ServletContextFactories.class.getName() }));

        }
        if(System.getProperty(Context.class.getName()) == null)
        {
            System.setProperty(Context.class.getName(),
                HttpSessionContext.class.getName());

            filterConfig.getServletContext().log(ServletFilterBundle.
                getSystemPropertyWarningMessage(Locale.getDefault()).
                format(new Object[] { Context.class.getName(),
                ServletContextFactories.class.getName() }));

        }
    }

    /**
     * Attaches the request's {@code HttpSession} with corresponding
     * {@code ServletContext} to the current thread of execution.
     *
     * @see #getServletContext()
     * @see #getHttpSession()
     */
    public void doFilter(final ServletRequest req,
        final ServletResponse rsp, final FilterChain chain) throws
        IOException, ServletException
    {

        try
        {
            if(req instanceof HttpServletRequest)
            {
                final HttpSession session = ((HttpServletRequest) req).
                    getSession(true);

                ServletFilter.CONTEXTS.set(session.getServletContext());
                ServletFilter.SESSIONS.set(session);
            }
            else
            {
                ServletFilter.CONTEXTS.set(this.servletContext);
                ServletFilter.SESSIONS.set(null);
            }

            chain.doFilter(req, rsp);
        }
        finally
        {
            // TODO JDK 1.5
            //ServletFilter.CONTEXTS.remove();
            //ServletFilter.SESSIONS.remove();

            ServletFilter.CONTEXTS.set(null);
            ServletFilter.SESSIONS.set(null);
        }
    }

    public void destroy()
    {
        this.servletContext = null;
    }

    //------------------------------------------------------------------Filter--
}
